#include <iostream>
#include <cstring>
#include <vector>
#include <algorithm>
#include <map>
#include "common.h"
using namespace std;

template <typename K, typename D>
static D unique_map(std::map<K,D>& map, vector<K>& v, const K key)
{
	typename std::map<K,D>::const_iterator p = map.find(key);
	if (p != map.end())
		return (*p).second;
	D nv = map[key] = v.size();
	v.push_back(key);
#ifdef DEBUG
	cerr << "UNIQ: '" << key << "' -> " << nv << endl;
#endif
	return nv;
}

#define FAIL_ON(b) if (!data_in || (b)) do { \
	cerr << "ERR: (" << #b << ")\n"; \
	return false; \
} while(0)

static bool read_data_common_start(zadanie& z, map<char, int>& charmap, istream& data_in)
{
	data_in >> z.rzad;
	FAIL_ON(z.rzad < 2);
	z.rr = z.rzad * z.rzad;
	z.chars.reserve(z.rr);

	data_in >> z.add_diag;
	FAIL_ON(!"add_diag read");
#if 0 && defined(DEBUG)
	const char *p = "123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
	while (*p && unique_map(charmap, z.chars, *p) < z.rr - 1)
		++p;
#endif

	return true;
}

static bool read_data_common_end(zadanie& z, map<char, int>& charmap, istream& data_in)
{
	const char *p = "123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
	FAIL_ON(z.chars.size() > (unsigned)z.rr);
	if (z.chars.size() < (unsigned)z.rr) {
		while (*p && unique_map(charmap, z.chars, *p) < z.rr - 1)
			++p;
	}
#ifdef DEBUG
	cerr << "chars: " << z.chars.size() << '/' << z.rr << endl;
#endif
	FAIL_ON(z.chars.size() < (unsigned)z.rr);

	return true;
}

bool read_data_form1(zadanie& z, istream& data_in)
{
	map<char, int> charmap;

	if (!read_data_common_start(z, charmap, data_in))
		return false;

	for(;;) {
		int x, y;
		char d = ' ';

		data_in >> x >> y;
		if (x == 0 && y == 0)
			break;

		FAIL_ON(x < 1 || x > z.rr || y < 1 || y > z.rr);
		--y, --x;

		do {
			data_in >> d;
			FAIL_ON(!"char read");
		} while (isspace(d));

		int v = unique_map(charmap, z.chars, d);

		struct init_pt ip = { x + y * z.rr, v };
		z.init.push_back(ip);
	}

	return read_data_common_end(z, charmap, data_in);
}

bool read_data_form2(zadanie& z, istream& data_in)
{
	map<char, int> charmap;

	if (!read_data_common_start(z, charmap, data_in))
		return false;

	char emptyd;
	do {
		data_in >> emptyd;
		FAIL_ON(!"empty char read");
	} while (isspace(emptyd));

	struct init_pt ip = { 0, 0 };
	for(int y = 0; y < z.rr; ++y) {
		for(int x = 0; x < z.rr; ++x) {
			char d;

			do {
				data_in >> d;
				FAIL_ON(!"char read");
			} while (isspace(d));

			if (d != emptyd) {
				ip.v = unique_map(charmap, z.chars, d);
				z.init.push_back(ip);
			}

			++ip.nr;
		}
	}

	return read_data_common_end(z, charmap, data_in);
}

bool write_data(const zadanie& z, const rozwiazanie& r, ostream& data_out)
{
	vector<int>::const_iterator p = r.begin();

	for(int y = 0; y < z.rr; ++y) {
		for(int x = 0; x < z.rr; ++x, ++p) {
			data_out << ((*p >= 0) ? z.chars[*p] : '?');
			if (((x+1) % z.rzad) == 0)
				data_out << ' ';
		}
		data_out << endl;
		if (((y+1) % z.rzad) == 0)
			data_out << endl;
	}

	return true;
}

